@@ -33,21 +33,13 @@
 using namespace base;
 using namespace thin_provisioning;
 
+#define METADATA_VERSION 2
+
 //----------------------------------------------------------------
 
 namespace {
 	using namespace superblock_detail;
 
-	unsigned const METADATA_CACHE_SIZE = 1024;
-
-	transaction_manager::ptr
-	open_tm(block_manager<>::ptr bm) {
-		space_map::ptr sm(new core_map(bm->get_nr_blocks()));
-		sm->inc(SUPERBLOCK_LOCATION);
-		transaction_manager::ptr tm(new transaction_manager(bm, sm));
-		return tm;
-	}
-
 	void
 	copy_space_maps(space_map::ptr lhs, space_map::ptr rhs) {
 		for (block_address b = 0; b < rhs->get_nr_blocks(); b++) {
@@ -66,10 +58,10 @@ metadata::metadata(block_manager<>::ptr bm, open_type ot,
 {
 	switch (ot) {
 	case OPEN:
-		tm_ = open_tm(bm);
+		tm_ = open_tm(bm, SUPERBLOCK_LOCATION);
 		sb_ = read_superblock(tm_->get_bm());
 
-		if (sb_.version_ != 1)
+		if (sb_.version_ < METADATA_VERSION)
 			throw runtime_error("unknown metadata version");
 
 		metadata_sm_ = open_metadata_sm(*tm_, &sb_.metadata_space_map_root_);
@@ -85,7 +77,7 @@ metadata::metadata(block_manager<>::ptr bm, open_type ot,
 		break;
 
 	case CREATE:
-		tm_ = open_tm(bm);
+		tm_ = open_tm(bm, SUPERBLOCK_LOCATION);
 		space_map::ptr core = tm_->get_sm();
 		metadata_sm_ = create_metadata_sm(*tm_, tm_->get_bm()->get_nr_blocks());
 		copy_space_maps(metadata_sm_, core);
@@ -101,30 +93,32 @@ metadata::metadata(block_manager<>::ptr bm, open_type ot,
 
 		::memset(&sb_, 0, sizeof(sb_));
 		sb_.magic_ = SUPERBLOCK_MAGIC;
-		sb_.version_ = 1;
+		sb_.version_ = METADATA_VERSION;
 		sb_.data_mapping_root_ = mappings_->get_root();
 		sb_.device_details_root_ = details_->get_root();
 		sb_.data_block_size_ = data_block_size;
 		sb_.metadata_block_size_ = MD_BLOCK_SIZE >> SECTOR_SHIFT;
-		sb_.metadata_nr_blocks_ = tm_->get_bm()->get_nr_blocks();
+		sb_.metadata_nr_blocks_ = metadata_sm_->get_nr_blocks();
 
 		break;
 	}
 }
 
-metadata::metadata(block_manager<>::ptr bm)
+metadata::metadata(block_manager<>::ptr bm, bool read_space_maps)
 {
-	tm_ = open_tm(bm);
+	tm_ = open_tm(bm, SUPERBLOCK_LOCATION);
 	sb_ = read_superblock(tm_->get_bm(), SUPERBLOCK_LOCATION);
 
-	open_space_maps();
+	if (read_space_maps)
+		open_space_maps();
+
 	open_btrees();
 }
 
 metadata::metadata(block_manager<>::ptr bm,
 		   boost::optional<block_address> metadata_snap)
 {
-	tm_ = open_tm(bm);
+	tm_ = open_tm(bm, SUPERBLOCK_LOCATION);
 
 	superblock_detail::superblock actual_sb = read_superblock(bm, SUPERBLOCK_LOCATION);
 
