@@ -80,16 +80,6 @@ namespace local {
 
 	//--------------------------------
 
-	transaction_manager::ptr
-	open_tm(block_manager<>::ptr bm) {
-		space_map::ptr sm(new core_map(bm->get_nr_blocks()));
-		sm->inc(superblock_detail::SUPERBLOCK_LOCATION);
-		transaction_manager::ptr tm(new transaction_manager(bm, sm));
-		return tm;
-	}
-
-	//--------------------------------
-
 	struct mapping {
 		mapping()
 			: vbegin_(0),
@@ -103,13 +93,6 @@ namespace local {
 			  len_(len) {
 		}
 
-		void consume(uint64_t delta) {
-			delta = min<uint64_t>(delta, len_);
-			vbegin_ += delta;
-			dbegin_ += delta;
-			len_ -= delta;
-		}
-
 		uint64_t vbegin_, dbegin_, len_;
 	};
 
@@ -120,6 +103,51 @@ namespace local {
 		return out;
 	}
 
+	//--------------------------------
+
+	template <typename Container>
+	class mapping_stream {
+	public:
+		mapping_stream(Container const &c)
+		: it_(c.begin()),
+		  end_(c.end()) {
+			m_ = *it_;
+		}
+
+		mapping const &get_mapping() const {
+			return m_;
+		}
+
+		bool more_mappings() const {
+			return it_ != end_;
+		}
+
+		void consume(uint64_t delta) {
+			if (it_ == end_)
+				throw runtime_error("end of stream already reached");
+
+			if (delta > m_.len_)
+				throw runtime_error("delta too long");
+
+			if (delta == m_.len_) {
+				++it_;
+				m_ = *it_;
+
+			} else {
+				m_.vbegin_ += delta;
+				m_.dbegin_ += delta;
+				m_.len_ -= delta;
+			}
+		}
+
+	private:
+		typename Container::const_iterator it_;
+		typename Container::const_iterator end_;
+		mapping m_;
+	};
+
+	//--------------------------------
+
 	typedef std::deque<mapping> mapping_deque;
 
 	// Builds up an in core rep of the mappings for a device.
@@ -426,57 +454,47 @@ namespace local {
 
 		// We iterate through both sets of mappings in parallel
 		// noting any differences.
-		mapping_deque::const_iterator left_it = left.begin();
-		mapping_deque::const_iterator right_it = right.begin();
+		mapping_stream<mapping_deque> ls{left};
+		mapping_stream<mapping_deque> rs{right};
 
-		mapping left_mapping;
-		mapping right_mapping;
+		while (ls.more_mappings() && rs.more_mappings()) {
+			auto &lm = ls.get_mapping();
+			auto &rm = rs.get_mapping();
 
-		while (left_it != left.end() && right_it != right.end()) {
-			if (!left_mapping.len_ && left_it != left.end())
-				left_mapping = *left_it++;
+			if (lm.vbegin_ < rm.vbegin_) {
+				auto delta = min<uint64_t>(lm.len_, rm.vbegin_ - lm.vbegin_);
+				e.left_only(lm.vbegin_, lm.dbegin_, delta);
+				ls.consume(delta);
 
-			if (!right_mapping.len_ && right_it != right.end())
-				right_mapping = *right_it++;
+			} else if (lm.vbegin_ > rm.vbegin_) {
+				auto delta = min<uint64_t>(rm.len_, lm.vbegin_ - rm.vbegin_);
+				e.right_only(rm.vbegin_, rm.dbegin_, delta);
+				rs.consume(delta);
 
-			while (left_mapping.len_ && right_mapping.len_) {
-				if (left_mapping.vbegin_ < right_mapping.vbegin_) {
-					uint64_t delta = min<uint64_t>(left_mapping.len_, right_mapping.vbegin_ - left_mapping.vbegin_);
-					e.left_only(left_mapping.vbegin_, left_mapping.dbegin_, delta);
-					left_mapping.consume(delta);
+			} else if (lm.dbegin_ != rm.dbegin_) {
+				auto delta = min<uint64_t>(lm.len_, rm.len_);
+				e.blocks_differ(lm.vbegin_, lm.dbegin_, rm.dbegin_, delta);
+				ls.consume(delta);
+				rs.consume(delta);
 
-				} else if (left_mapping.vbegin_ > right_mapping.vbegin_) {
-					uint64_t delta = min<uint64_t>(right_mapping.len_, left_mapping.vbegin_ - right_mapping.vbegin_);
-					e.right_only(right_mapping.vbegin_, right_mapping.dbegin_, delta);
-					right_mapping.consume(delta);
-
-				} else if (left_mapping.dbegin_ != right_mapping.dbegin_) {
-					uint64_t delta = min<uint64_t>(left_mapping.len_, right_mapping.len_);
-					e.blocks_differ(left_mapping.vbegin_, left_mapping.dbegin_, right_mapping.dbegin_, delta);
-					left_mapping.consume(delta);
-					right_mapping.consume(delta);
-
-				} else {
-					uint64_t delta = min<uint64_t>(left_mapping.len_, right_mapping.len_);
-					e.blocks_same(left_mapping.vbegin_, left_mapping.dbegin_, delta);
-					left_mapping.consume(delta);
-					right_mapping.consume(delta);
-				}
+			} else {
+				auto delta = min<uint64_t>(lm.len_, rm.len_);
+				e.blocks_same(lm.vbegin_, lm.dbegin_, delta);
+				ls.consume(delta);
+				rs.consume(delta);
 			}
 		}
 
-		while (left_it != left.end()) {
-			left_mapping = *left_it++;
-
-			if (left_mapping.len_)
-				e.left_only(left_mapping.vbegin_, left_mapping.dbegin_, left_mapping.len_);
+		while (ls.more_mappings()) {
+			auto &lm = ls.get_mapping();
+			e.left_only(lm.vbegin_, lm.dbegin_, lm.len_);
+			ls.consume(lm.len_);
 		}
 
-		while (right_it != right.end()) {
-			right_mapping = *right_it++;
-
-			if (right_mapping.len_)
-				e.right_only(right_mapping.vbegin_, right_mapping.dbegin_, right_mapping.len_);
+		while (rs.more_mappings()) {
+			auto &rm = rs.get_mapping();
+			e.right_only(rm.vbegin_, rm.dbegin_, rm.len_);
+			rs.consume(rm.len_);
 		}
 
 		e.complete();
@@ -527,7 +545,7 @@ namespace local {
 		mapping_recorder mr2;
 		damage_visitor damage_v;
 		superblock_detail::superblock sb;
-		checked_space_map::ptr data_sm;
+		block_address nr_data_blocks = 0ull;
 
 		{
 			block_manager<>::ptr bm = open_bm(*fs.dev, block_manager<>::READ_ONLY, !fs.use_metadata_snap);
@@ -562,13 +580,16 @@ namespace local {
 
 			btree_visit_values(snap2, mr2, damage_v);
 			mr2.complete();
+
+			if (md->data_sm_)
+				nr_data_blocks = md->data_sm_->get_nr_blocks();
 		}
 
 		indented_stream is(cout);
 		begin_superblock(is, "", sb.time_,
 				 sb.trans_id_,
 				 sb.data_block_size_,
-				 data_sm->get_nr_blocks(),
+				 nr_data_blocks,
 				 sb.metadata_snap_ ?
 				 boost::optional<block_address>(sb.metadata_snap_) :
 				 boost::optional<block_address>());
@@ -631,7 +652,8 @@ thin_delta_cmd::run(int argc, char **argv)
 		{ "thin2", required_argument, NULL, 2 },
 		{ "snap2", required_argument, NULL, 2 },
 		{ "metadata-snap", optional_argument, NULL, 'm' },
-		{ "verbose", no_argument, NULL, 4 }
+		{ "verbose", no_argument, NULL, 4 },
+		{ NULL, no_argument, NULL, 0 }
 	};
 
 	while ((c = getopt_long(argc, argv, shortopts, longopts, NULL)) != -1) {
